/*
 * AboutDialog.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.imagecropper;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.underworldlabs.imagecropper.util.GUIUtilities;
import org.underworldlabs.swing.LinkButton;
import org.underworldlabs.swing.SystemPropertiesTable;

import edu.stanford.ejalbert.BrowserLauncher;
import edu.stanford.ejalbert.BrowserLauncherRunner;
import edu.stanford.ejalbert.exception.BrowserLaunchingInitializingException;
import edu.stanford.ejalbert.exception.UnsupportedOperatingSystemException;

/** 
 * Simple about dialog.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1344 $
 * @date     $Date: 2008-07-12 23:03:54 +1000 (Sat, 12 Jul 2008) $
 */
public class AboutDialog extends JDialog {
    
    /** Creates a new instance of AboutDialog */
    public AboutDialog() {
        super(GUIUtilities.getApplicationFrame());
        try {
            init();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void init() throws Exception {

        setTitle("About");

        JPanel aboutPanel = new JPanel(new GridBagLayout());
        ImageIcon icon = GUIUtilities.loadIcon("ImageCropper72.png");

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets.bottom = 5;
        gbc.insets.left = 5;
        gbc.insets.right = 5;
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        aboutPanel.add(new JLabel(icon), gbc);
        
        StringBuffer sb = new StringBuffer();
        sb.append("<html><table border=\"0\" cellspacing=\"0\" ");
        sb.append("cellpadding=\"0\"><tr><td align=\"center\"><h2>");
        sb.append("Image Cropper");
        sb.append("</h2><h3>");
        sb.append("0.1");
        sb.append("</h3>");
        sb.append("<p>&copy; Copyright Takis Diakoumis ");
        sb.append(java.util.Calendar.getInstance().get(java.util.Calendar.YEAR));
        sb.append("</p></td</tr></table></html>");
        
        gbc.gridx = 1;
        gbc.weighty = 0;
        gbc.fill = GridBagConstraints.BOTH;
        aboutPanel.add(new JLabel(sb.toString()), gbc);

        LinkButton linkButton = new LinkButton("http://underworldlabs.org");
        linkButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                launch("http://underworldlabs.org");
            }
        });
        
        gbc.gridy++;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.NONE;
        aboutPanel.add(linkButton, gbc);
        
        JPanel envPanel = new JPanel(new BorderLayout());
        envPanel.add(new JScrollPane(new SystemPropertiesTable()), BorderLayout.CENTER);
        
        JTabbedPane tabs = new JTabbedPane();
        tabs.add(aboutPanel, "About");
        tabs.add(envPanel, "Environment");
        
        tabs.setPreferredSize(new Dimension(450, 300));
        
        setResizable(false);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(tabs, BorderLayout.CENTER);

        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        
        pack();
        setLocation(GUIUtilities.getLocationForDialog(getSize()));
        setVisible(true);
    }
    
    public static void launch(String url) {
        try {
            BrowserLauncher launcher = new BrowserLauncher(null);
            BrowserLauncherRunner runner = 
                    new BrowserLauncherRunner(launcher, url, null);
            Thread launcherThread = new Thread(runner);
            launcherThread.start();
        } 
        catch (BrowserLaunchingInitializingException e) {
            handleException(e);
        }
        catch (UnsupportedOperatingSystemException e) {
            handleException(e);
        }
    }
    
    private static void handleException(Throwable e) {
        GUIUtilities.displayExceptionErrorDialog(
                "Error launching local web browser:\n" + 
                e.getMessage(), e);
    }

    private static final long serialVersionUID = 1L;
}
