package org.underworldlabs.imagecropper;

import java.io.IOException;

import org.underworldlabs.imagecropper.util.GUIUtilities;
import org.underworldlabs.util.FileUtils;

public abstract class AbstractImageCropProcessor implements ImageCropProcessor {

    public void restoreFromBackup(ImageFile imageFile) {
        
        try {
        
            FileUtils.copyFile(backupFileForImage(imageFile), imageFile.getAbsolutePath());

        } catch (IOException e) {

            throw new ImageCropProcessorException(e);
        }
            
    }
    
    protected final void handleError(Throwable e) {
        
        throw new ImageCropProcessorException(e);
    }

    protected final void createBackup(ImageFile imageFile) {
        
        try {

            FileUtils.copyFile(imageFile.getAbsolutePath(), backupFileForImage(imageFile));

        } catch (IOException e) {

            GUIUtilities.outputErrorText("Error creating backup for file " 
                    + imageFile.getName() + " - " + e.getMessage());
        }

    }

    protected final String backupFileForImage(ImageFile imageFile) {

        return System.getProperty("java.io.tmpdir") + "/" + imageFile.getName();
    }

    
}
