package org.underworldlabs.imagecropper;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;

public class DefaultImageCropProcessor extends AbstractImageCropProcessor {

    @Override
    public int crop(ImageFile imageFile, CropValues cropValues) {

        createBackup(imageFile);

        try {
        
            BufferedImage image = ImageIO.read(imageFile.getFile());
            
            Rectangle clip = cropValuesAsRectangle(image, cropValues);
            BufferedImage clipped = image.getSubimage(clip.x, clip.y, clip.width, clip.height);

            ImageIO.write(clipped, "JPG", imageFile.getFile());
            
        } catch (IOException e) {
            
            handleError(e);
        }
        
        return 0;
    }

    private Rectangle cropValuesAsRectangle(BufferedImage image, CropValues cropValues) {

        int x = cropValues.left;
        int y = cropValues.top;

        int width = image.getWidth() - cropValues.left - cropValues.right;   
        int height = image.getHeight() - cropValues.top - cropValues.bottom;   
        
        return new Rectangle(x, y, width, height);
    }
    
}
