package org.underworldlabs.imagecropper;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.underworldlabs.imagecropper.gui.DefaultButton;
import org.underworldlabs.imagecropper.gui.DefaultComboBox;
import org.underworldlabs.imagecropper.gui.GUIConstants;
import org.underworldlabs.imagecropper.util.GUIUtilities;
import org.underworldlabs.imagecropper.util.SystemResources;
import org.underworldlabs.swing.DynamicComboBoxModel;

public class DirectorySelectionPanel extends JPanel {

    private List<ImageDirectoryChangedListener> listeners;
    
    private final JComboBox folderNameField;

    private DynamicComboBoxModel folderNameModel;
    
    public DirectorySelectionPanel() {

        super(new GridBagLayout());

        BrowseForFolderAction action = new BrowseForFolderAction();

        folderNameModel = new DynamicComboBoxModel(
                SystemResources.getSearchQueryHistory());
        folderNameField = new DefaultComboBox(folderNameModel);
        folderNameField.setEditable(true);
        folderNameField.addActionListener(folderNameFieldActionListener());

		JButton button = new DefaultButton(action);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.insets.top = 13;
        gbc.insets.left = 2;
        gbc.insets.bottom = 10;
        gbc.gridy = 0;
        gbc.gridx = 0;
        add(new JLabel("Directory:"), gbc);
        gbc.gridx++;
        gbc.insets.top = 10;
        gbc.insets.left = 5;
        gbc.weightx = 1.0;
        gbc.ipady = 5;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(folderNameField, gbc);
        gbc.gridx++;
        gbc.weightx = 0;
        gbc.insets.right = 2;
        gbc.ipady = 0;
        gbc.fill = GridBagConstraints.NONE;
        add(button, gbc);

    }

	private ActionListener folderNameFieldActionListener() {

		return new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				
				File file = new File(folderNameText());
				if (directoryIsValid(file)) {
				
					fireImageDirectoryChanged(file);
				}

			}
		};

	}

	private boolean directoryIsValid(File file) {

		if (file.exists() && file.isDirectory()) {
		
			return true;
		
		} else {
			
			GUIUtilities.displayErrorMessage("The directory path entered is invalid");
			return false;
		}
	}
	
    public void addImageDirectoryChangeListener(ImageDirectoryChangedListener listener) {
        
        if (listeners == null) {
            
            listeners = new ArrayList<ImageDirectoryChangedListener>();
        }

        listeners.add(listener);
    }
    
    public void fireImageDirectoryChanged(File file) {

        if (listeners != null) {

            ImageDirectoryChangedEvent event = new ImageDirectoryChangedEvent(file.getAbsolutePath());
            for (ImageDirectoryChangedListener listener : listeners) {
                
                listener.imageDirectoryChanged(event);
            }
            
        }
        
    }

    @Override
    public Dimension getPreferredSize() {

        Dimension preferredSize = super.getPreferredSize();
        preferredSize.setSize(preferredSize.width, GUIConstants.DEFAULT_WIDGET_HEIGHT + 10);

        return preferredSize;
    }
    
    public String getFolderNameValue() {

        return folderNameText();
    }
    
    private String folderNameText() {

        Object selectedItem = folderNameModel.getSelectedItem();
        
        if (selectedItem != null) {
        
            return selectedItem.toString();
        }
        
        return "";
    }


    class BrowseForFolderAction extends AbstractAction {

        public BrowseForFolderAction() {

            putValue(Action.NAME, "Browse");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('o'));
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {

            JFileChooser fileChooser = new JFileChooser(getFolderNameValue());
            fileChooser.setDialogTitle("Select Image Directory");
            fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

            int result = fileChooser.showDialog(GUIUtilities.getApplicationFrame(), "Select");
            if (result != JFileChooser.CANCEL_OPTION) {
             
                File file = fileChooser.getSelectedFile();
                folderNameModel.addElement(file.getAbsolutePath());
                fireImageDirectoryChanged(file);
            }
            
        }
        
    }

}
