package org.underworldlabs.imagecropper;

import java.io.File;
import java.util.Arrays;
import java.util.Iterator;

import org.underworldlabs.imagecropper.util.GUIUtilities;

public class ImageDirectoryIterator implements Iterator<ImageFile> {

    private String directoryLocation;
    
    private int currentIndex;
    
    private File[] imageFiles;

    @Override
    public boolean hasNext() {

        return imageFiles != null 
            && imageFiles.length > 0 
            && currentIndex < imageFiles.length - 1;
    }

    public boolean hasPrevious() {

        return imageFiles != null 
            && imageFiles.length > 0 
            && currentIndex > 0;
    }

    @Override
    public ImageFile next() {

        if (hasNext()) {
        
            return new ImageFile(imageFiles[++currentIndex]);
        }

        return null;
    }

    @Override
    public void remove() {

        // not interested
    }

    public int getCurrentIndex() {
        
        return currentIndex;
    }
    
    public void imageDirectoryChanged(String directoryLocation) {

        this.directoryLocation = directoryLocation;
        loadImages();
        
        log("Setting image directory path to " + directoryLocation);        
    }

    private void loadImages() {

        currentIndex = -1;
        imageFiles = new File(directoryLocation).listFiles();

        log("Found " + imageFiles.length + " files in directory");
        
        Arrays.sort(imageFiles);
    }

	public ImageFile previous() {

		if (hasPrevious() && currentIndex > 0) {

			return new ImageFile(imageFiles[--currentIndex]);
		}
		
		return null;
	}

	private void log(String text) {
	    
	    GUIUtilities.outputPlainText(text);
	}
	
}
