package org.underworldlabs.imagecropper;

import java.io.IOException;

import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.exec.ExecuteException;
import org.apache.commons.exec.ExecuteWatchdog;
import org.underworldlabs.imagecropper.util.GUIUtilities;

public class ImageMagickImageCropProcessor extends AbstractImageCropProcessor {

    private static final String MOGRIFY = "gm mogrify -quality 100 -crop ";
    
    public int crop(ImageFile imageFile, CropValues cropValues) {
        
        createBackup(imageFile);
        
        String commandString = MOGRIFY + leftAndBottom(cropValues) + imageFile.getAbsolutePath();

        int exitValue = execute(commandString);
        if (exitValue == 0) {
            
            commandString = MOGRIFY + rightAndTop(cropValues) + imageFile.getAbsolutePath();
            exitValue = execute(commandString);
        }
        
        return exitValue;
    }

    private int execute(String commandString) {

        GUIUtilities.outputPlainText("Executing:   " + commandString);
        
        CommandLine commandLine = CommandLine.parse(commandString);
        
        DefaultExecutor executor = new DefaultExecutor();
        executor.setExitValue(0);

        ExecuteWatchdog watchdog = new ExecuteWatchdog(60000);
        executor.setWatchdog(watchdog);

        int exitValue = -1;

        try {

            exitValue = executor.execute(commandLine);

        } catch (ExecuteException e) {

            handleError(e);

        } catch (IOException e) {

            handleError(e);
        }

        return exitValue;
    }
    
    private String rightAndTop(CropValues cropValues) {
        
        String cropString = "";
        
        if (cropValues.right > 0) {
        
            cropString += "-" + cropValues.right;
            
        } else {
            
            cropString += "+0";
        }
            
        cropString += "+" + cropValues.top;
        
        return cropString + " ";
    }
    
    private String leftAndBottom(CropValues cropValues) {
        
        String cropString = "+" + cropValues.left;

        if (cropValues.bottom > 0) {
            
            cropString += "-" + cropValues.bottom;

        } else {
            
            cropString += "+0";
        }
        
        return cropString + " ";
    }
    
}
