package org.underworldlabs.imagecropper;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.DecimalFormat;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.underworldlabs.imagecropper.gui.DefaultButton;
import org.underworldlabs.imagecropper.gui.DefaultTextField;
import org.underworldlabs.imagecropper.gui.TextFieldPanel;
import org.underworldlabs.imagecropper.util.GUIUtilities;
import org.underworldlabs.swing.plaf.UIUtils;

public class ImageOptionsPanel extends JPanel implements ImageSelectionChangedListener {

    private JLabel imageNameLabel;
    
    private JLabel imageSizeLabel;
    
    private JLabel imageFileSizeLabel;
    
    private JTextField cropTopField;
    
    private JTextField cropBottomField;
    
    private JTextField cropLeftField;
    
    private JTextField cropRightField;
    
    private PreviewPanel previewPanel;
    
    private DecimalFormat decimalFormat = new DecimalFormat("###.0 KB");
    
    private ImageCropProcessorDelegate imageCropProcessorDelegate;
    
    public ImageOptionsPanel() {

        super(new GridBagLayout());

        JButton undoButton = new DefaultButton(new UndoAction());
        JButton executeButton = new DefaultButton(new ExecuteAction());
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.gridwidth = 2;
        gbc.insets.bottom = 5;
        gbc.insets.left = 5;
        gbc.insets.right = 5;
        gbc.insets.top = 5;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        add(createMetaDataPanel(), gbc);
        gbc.gridy++;
        add(createCropFieldsPanel(), gbc);
        gbc.gridwidth = 1;
        gbc.gridy++;
        gbc.weightx = 0.5;
        add(undoButton, gbc);
        gbc.gridx = 1;
        gbc.weightx = 0.5;
        add(executeButton, gbc);
        
        previewPanel = new PreviewPanel();
        previewPanel.setBorder(BorderFactory.createTitledBorder("Preview"));
        gbc.gridwidth = 2;
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weighty = 1.0;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        add(previewPanel, gbc);

        setBorder(BorderFactory.createLineBorder(UIUtils.getDefaultBorderColour(), 1));
    }

    public void loadPreviewImage(ImageFile imageFile) {
        
        previewPanel.update(imageFile);
    }
    
    public void registerImageCropProcessorDelegate(ImageCropProcessorDelegate imageCropProcessorDelegate) {

        this.imageCropProcessorDelegate = imageCropProcessorDelegate;
    }
    
    @Override
    public void imageSelectionChanged(ImageFile imageFile) {

        imageNameLabel.setText(imageFile.getName());
        imageSizeLabel.setText(imageFile.getWidth() + " x " + imageFile.getHeight());
        imageFileSizeLabel.setText(formatFileSize(imageFile));
        
        JLabel[] jLabels = updateableLabelsAsArray();
        for (JLabel label : jLabels) {
            
            label.paintImmediately(0, 0, label.getWidth(), label.getHeight());
        }

    }

    private String formatFileSize(ImageFile imageFile) {
        
        return decimalFormat.format((double) imageFile.getSize() / 1024);
    }    
    
    @Override
    public Dimension getPreferredSize() {

        Dimension preferredSize = super.getPreferredSize();
        preferredSize.setSize(320, preferredSize.height);

        return preferredSize;
    }

    private JPanel createMetaDataPanel() {
        
        JPanel panel = new JPanel(new GridBagLayout());
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.insets.top = 5;
        gbc.insets.bottom = 5;
        gbc.insets.left = 5;
        gbc.anchor = GridBagConstraints.NORTHWEST;

        String[] labels = {"Name:", "Dimensions:", "Size on Disk:"};
        
        for (String label : labels) {
            
            panel.add(new JLabel(label), gbc);
            gbc.gridy++;
        }
        
        gbc.gridy = 0;
        gbc.gridx = 1;
        gbc.weightx = 1.0;
        gbc.insets.right = 5;
        gbc.insets.left = 10;
        gbc.fill = GridBagConstraints.HORIZONTAL;

        imageNameLabel = new JLabel();
        imageSizeLabel = new JLabel();
        imageFileSizeLabel = new JLabel();

        JLabel[] jLabels = updateableLabelsAsArray();

        for (JLabel label : jLabels) {
            
            panel.add(label, gbc);
            gbc.gridy++;
        }
        
        panel.setBorder(BorderFactory.createTitledBorder("Image"));
        
        return panel;
    }

    private JLabel[] updateableLabelsAsArray() {

        return new JLabel[]{imageNameLabel, imageSizeLabel, imageFileSizeLabel};
    }

    private JPanel createCropFieldsPanel() {
        
        JPanel panel = new TextFieldPanel(new GridBagLayout());
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.insets.top = 3;
        gbc.insets.bottom = 5;
        gbc.insets.left = 5;
        gbc.anchor = GridBagConstraints.NORTHWEST;

        String[] labels = {"Top:", "Bottom:", "Left:", "Right:"};
        
        for (String label : labels) {
            
            panel.add(new JLabel(label), gbc);
            gbc.gridy++;
        }
        
        gbc.gridy = 0;
        gbc.gridx = 1;
        gbc.ipady = 5;
        gbc.weightx = 1.0;
        gbc.insets.right = 5;
        gbc.insets.left = 10;
        gbc.insets.top = 0;
        gbc.fill = GridBagConstraints.HORIZONTAL;

        cropTopField = new DefaultTextField();
        cropBottomField = new DefaultTextField();
        cropLeftField = new DefaultTextField();
        cropRightField = new DefaultTextField();
        
        JTextField[] fields = cropFieldsAsArray();

        Action action = new ExecuteAction();
        
        for (JTextField field : fields) {
            
            field.setAction(action);
            
            panel.add(field, gbc);
            gbc.gridy++;
        }
        
        panel.setBorder(BorderFactory.createTitledBorder("Crop"));
        
        return panel;
    }

    private JTextField[] cropFieldsAsArray() {

        return new JTextField[]{cropTopField, cropBottomField, cropLeftField, cropRightField};
    }

    private boolean cropValuesValid() {
        
        JTextField[] fields = cropFieldsAsArray();

        for (JTextField field : fields) {

            String text = field.getText();
            
            if (StringUtils.isNotBlank(text) && !StringUtils.isNumeric(text)) {
                
                GUIUtilities.displayErrorMessage("Invalid crop values");
                return false;
            }
            
        }

        return true;
    }
    
    private CropValues cropValues() {

        return new CropValues(
                valueOrZero(cropTopField), 
                valueOrZero(cropBottomField),
                valueOrZero(cropLeftField),
                valueOrZero(cropRightField));
    }
    
    private int valueOrZero(JTextField field) {

        String text = field.getText();
        if (StringUtils.isNotBlank(text) && StringUtils.isNumeric(text)) {
            
            return Integer.valueOf(text);
        }
        
        return 0;
    }

    
    class PreviewPanel extends JPanel {
        
        private JLabel previewImageLabel;
        
        public PreviewPanel() {

            super(new GridBagLayout());
            
            previewImageLabel = new JLabel();
            add(previewImageLabel, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
                    GridBagConstraints.SOUTHEAST, 
                    GridBagConstraints.BOTH,
                    new Insets(5, 5, 5, 5), 0, 0));
        }
        
        void update(ImageFile imageFile) {
            
            previewImageLabel.setIcon(scaleImageToFit(imageFile));
            previewImageLabel.revalidate();

            previewImageLabel.paintImmediately(0, 0, previewImageLabel.getWidth(), previewImageLabel.getHeight());
        }
        
        private ImageIcon scaleImageToFit(ImageFile imageFile) {
            
            try {
                
                BufferedImage image = ImageIO.read(imageFile.getFile());
                
                int width = previewImageLabel.getWidth();
                int height = previewImageLabel.getHeight();

                Image scaledInstance = image.getScaledInstance(width, height, Image.SCALE_SMOOTH);

                return new ImageIcon(scaledInstance);
                
            } catch (IOException e) {

                throw new RuntimeException(e);
            }
            
        }
        
        
    }
    
    
    class ExecuteAction extends AbstractAction {
        
        public ExecuteAction() {

            putValue(Action.NAME, "Execute");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('E'));
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {

            if (cropValuesValid()) {
                
                imageCropProcessorDelegate.doCrop(cropValues());                
            }
            
        }
        
    }

    
    class UndoAction extends AbstractAction {
        
        public UndoAction() {

            putValue(Action.NAME, "Undo");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('U'));
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {

            imageCropProcessorDelegate.undoCrop();
        }
        
    }

    
}
