package org.underworldlabs.imagecropper;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;

import org.underworldlabs.imagecropper.gui.DefaultButton;
import org.underworldlabs.imagecropper.util.GUIUtilities;
import org.underworldlabs.swing.FlatSplitPane;
import org.underworldlabs.swing.GUIUtils;

public class ImageViewBasePanel extends JPanel {

    private ImageViewPanel topImageViewPanel;
    
    private ImageViewPanel bottomImageViewPanel;
    
    private ScrollingButtonsPanel scrollingButtonsPanel;
    
    public ImageViewBasePanel() {

        super(new BorderLayout());
        
        topImageViewPanel = new ImageViewPanel();
        bottomImageViewPanel = new ImageViewPanel() {
            protected void fireImageSelectionChanged(ImageFile imageFile) {};
            protected void resetScrollPaneView() {
                scrollPane.getHorizontalScrollBar().setValue(0);
                scrollPane.getVerticalScrollBar().setValue(
                        scrollPane.getVerticalScrollBar().getMaximum());
            }
        };
        
        FlatSplitPane splitPane = new FlatSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setDividerSize(4);
        splitPane.setResizeWeight(0.5);

        splitPane.setTopComponent(topImageViewPanel);
        splitPane.setBottomComponent(bottomImageViewPanel);
        
        add(splitPane, BorderLayout.CENTER);
        
        scrollingButtonsPanel = new ScrollingButtonsPanel();
        add(scrollingButtonsPanel, BorderLayout.SOUTH);
    }
    
    public void load(ImageFile imageFile) {

        topImageViewPanel.prepare();
        bottomImageViewPanel.prepare();
        
        topImageViewPanel.load(imageFile);
        bottomImageViewPanel.load(imageFile);

        GUIUtils.invokeLater(new Runnable() {
            @Override
            public void run() {

                bottomImageViewPanel.resetScrollPaneView();
            }
        });
        
    }
    
    public void addImageSelectionChangingListener(ImageSelectionChangingListener listener) {
        
        topImageViewPanel.addImageSelectionChangingListener(listener);
    }
    
    public void addImageSelectionChangedListener(ImageSelectionChangedListener listener) {
        
        topImageViewPanel.addImageSelectionChangedListener(listener);
    }
    
    public void setNextButtonEnabled(boolean enable) {
        
        scrollingButtonsPanel.setNextButtonEnabled(enable);
    }

    public void setPreviousButtonEnabled(boolean enable) {
        
        scrollingButtonsPanel.setPreviousButtonEnabled(enable);
    }

    private void fireNextImageSelected() {

        topImageViewPanel.fireNextImageSelected();
    }
    
    private void firePreviousImageSelected() {
        
        topImageViewPanel.firePreviousImageSelected();
    }


    class ScrollingButtonsPanel extends JPanel {
        
        private JButton previousButton;

        private JButton nextButton;

        public ScrollingButtonsPanel() {

            super(new GridBagLayout());

            previousButton = new ImageScrollingButton(new PreviousImageAction());
            nextButton = new ImageScrollingButton(new NextImageAction());

            GridBagConstraints gbc = new GridBagConstraints();
            gbc.gridx = 0;
            gbc.insets.top = 5;
            gbc.weightx = 1.0;
            gbc.anchor = GridBagConstraints.EAST;
            add(previousButton, gbc);
            gbc.insets.left = 5;
            gbc.gridx++;
            gbc.weightx = 0;
            add(nextButton, gbc);
        }
        
        public void setNextButtonEnabled(boolean enable) {
            
            nextButton.setEnabled(enable);
        }

        public void setPreviousButtonEnabled(boolean enable) {
            
            previousButton.setEnabled(enable);
        }
        
    }

    
    class ImageScrollingButton extends DefaultButton {
        
        public ImageScrollingButton(Action action) {
            
            super(action);
        }
        
        @Override
        public Dimension getPreferredSize() {

            Dimension preferredSize = super.getPreferredSize();
            preferredSize.setSize(150, preferredSize.height);

            return preferredSize;
        }
        
    }

    private void registerAction(String actionId, Action action) {

        GUIUtilities.getActionMap().put(actionId, action);
        GUIUtilities.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(
                (KeyStroke) action.getValue(Action.ACCELERATOR_KEY), actionId);
    }

 
    class NextImageAction extends AbstractAction {
        
        public NextImageAction() {

            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, 0));
            putValue(Action.NAME, "Next");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('x'));
            
            registerAction("next-image", this);
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            fireNextImageSelected();
        }
        
    }
    
    class PreviousImageAction extends AbstractAction {
        
        public PreviousImageAction() {

            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, 0));
            putValue(Action.NAME, "Previous");
            putValue(Action.MNEMONIC_KEY, Integer.valueOf('P'));
            
            registerAction("previous-image", this);
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {

            firePreviousImageSelected();
        }
        
    }

}
