package org.underworldlabs.imagecropper;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.underworldlabs.swing.plaf.UIUtils;

public class ImageViewPanel extends JPanel {

    private JLabel imageContent;
    
    private List<ImageSelectionChangedListener> changedListeners;
    
    private List<ImageSelectionChangingListener> changingListeners;

    protected JScrollPane scrollPane;
    
    public ImageViewPanel() {

        setLayout(new BorderLayout());
        
        imageContent = new JLabel();
        imageContent.setHorizontalAlignment(JLabel.CENTER);

        scrollPane = new JScrollPane(imageContent);
        add(scrollPane, BorderLayout.CENTER);
        
        setBorder(BorderFactory.createLineBorder(UIUtils.getDefaultBorderColour(), 1));
    }

    public void addImageSelectionChangingListener(ImageSelectionChangingListener listener) {
        
        if (changingListeners == null) {
            
            changingListeners = new ArrayList<ImageSelectionChangingListener>();
        }

        changingListeners.add(listener);
    }
    
    public void addImageSelectionChangedListener(ImageSelectionChangedListener listener) {
        
        if (changedListeners == null) {
            
            changedListeners = new ArrayList<ImageSelectionChangedListener>();
        }

        changedListeners.add(listener);
    }
    
    public void prepare() {
        
        imageContent.setVisible(false);
    }
    
    public void load(ImageFile imageFile) {
        
        try {
        
            imageContent.setIcon(imageFile.getImage());
            imageContent.revalidate();

            imageContent.paintImmediately(0, 0, imageContent.getWidth(), imageContent.getHeight());
            
            fireImageSelectionChanged(imageFile);
            
        } finally {
            
            imageContent.setVisible(true);
        }
    }
    
    protected void fireNextImageSelected() {
    	
    	if (changingListeners != null) {
    		
    		for (ImageSelectionChangingListener listener : changingListeners) {
    			
    			listener.scrollForwards();
    		}
    		
    	}

    	resetScrollPaneView();
    }

    protected void firePreviousImageSelected() {
    	
    	if (changingListeners != null) {
    		
    		for (ImageSelectionChangingListener listener : changingListeners) {
    			
    			listener.scrollBackwards();
    		}
    		
    	}
    	
    	resetScrollPaneView();
    }

    protected void fireImageSelectionChanged(ImageFile imageFile) {
        
        if (changedListeners != null) {
            
            for (ImageSelectionChangedListener listener : changedListeners) {
                
                listener.imageSelectionChanged(imageFile);
            }
            
        }
        
    }
    
    protected void resetScrollPaneView() {

        scrollPane.getHorizontalScrollBar().setValue(0);
        scrollPane.getVerticalScrollBar().setValue(0);
    }

}
